//Normal mapping shader, made by TDRR.
uniform sampler2D texture2;

#define PI 3.1415926535897932384626433832795

//comment out this line if in your normal maps, green points up
#define NORMALMAP_GREEN_DOWN 1

vec4 BrightmapProcess(vec4 color, vec4 texel, vec4 brightpix)
{
	vec4 new_brightpix = desaturate(brightpix);
	return vec4(texel.rgb * min (color.rgb + new_brightpix.rgb, 1.0), texel.a*color.a);
}

vec3 specularity (vec3 lightPos, vec3 viewDir, vec3 normal, vec3 color)
{	
	vec3 lightDir = normalize(lightPos - pixelpos.xyz);  
	
	vec3 reflectDir = reflect(-lightDir, normal);
	
	vec3 spec = vec3(pow(max(dot(viewDir, reflectDir), 0.0), 32));
	return spec * color;
}

vec4 Process(vec4 color)
{
	vec2 UV = gl_TexCoord[0].st;
	vec4 normalmap = texture2D(texture2, UV);
	vec4 texel = getTexel(UV);
	
	if(normalmap.a > 0.98) {BrightmapProcess(color, texel, normalmap);}
	
	vec3 eyedir = normalize(camerapos.xyz - pixelpos.xyz);
    vec3 normal = normalize(cross(dFdx(pixelpos.xyz), dFdy(pixelpos.xyz)));
	
	vec3 nmap = vec3(0.0);
	if(normal.y > 0.05) //ceiling normal
	{
		nmap = normalize(normalmap.xzy * 2.0 - 1.0);
		nmap.x = -nmap.x;
		#ifdef NORMALMAP_GREEN_DOWN
			nmap.z = -nmap.z;
		#endif
	}
	else if(normal.y < -0.05) //below 0.05, it's a floor
	{
		nmap = normalize(normalmap.xzy * 2.0 - 1.0);
		nmap = -nmap;
		#ifdef NORMALMAP_GREEN_DOWN
			nmap = -nmap;
		#else
			nmap.x = -nmap.x;
			nmap.z = -nmap.z;
		#endif
	}
	else {return texel * color;} //probably a wall, don't do anything
	
	vec3 spec = vec3(0.0);
	
	#ifdef DYNLIGHT
		for(int i=0; i<lightrange.x; i+=2)
		{
			vec4 lightpos = lights[i];
			vec4 lightcolor = lights[i+1];
			
			lightcolor.rgb *= max(lightpos.w - distance(pixelpos.xyz, lightpos.xyz),0.0) / lightpos.w;
			spec += specularity(lightpos.xyz, eyedir, nmap, lightcolor.xyz);
		}
	#endif
	
	vec3 lightPos = vec3(0.0, -64.0, 0.0);
	
	float lightLevel = 1.0; //0.4 + 0.6 * clamp(dot(lightPos, nmap), 0.0, 1.0);
	
	//texel = vec4(0.25, 0.25, 0.25, 1.0);
	
	//vec4 envmap = texture2D(tex, reflect(eyedir, nmap).xz);
	//vec3 spec = specularity(lightPos, eyedir, nmap);
	return vec4( (texel.rgb * color.rgb * lightLevel) + spec, color.a);
	//return nmap * 0.5 + 0.5;
}